@@ -56,6 +56,26 @@ test('Unknown CI', function (t) {
   t.end()
 })
 
+test('Anonymous CI', function (t) {
+  const ANONYMOUS_ENV_VARS = [
+    'CI', 'CONTINUOUS_INTEGRATION', 'BUILD_NUMBER',
+    'CI_APP_ID', 'CI_BUILD_ID', 'CI_BUILD_NUMBER',
+    'RUN_ID'
+  ]
+
+  for (const envVar in ANONYMOUS_ENV_VARS) {
+    process.env[envVar] = true
+
+    clearModule('./')
+    const ci = require('./')
+    t.equal(ci.isCI, true)
+    t.equal(ci.isPR, null)
+    t.equal(ci.name, null)
+  }
+
+  t.end()
+})
+
 test('AppVeyor - PR', function (t) {
   process.env.APPVEYOR = 'true'
   process.env.APPVEYOR_PULL_REQUEST_NUMBER = '42'
@@ -746,7 +766,96 @@ test('Visual Studio App Center', function (t) {
   t.equal(ci.APPCENTER, true)
   assertVendorConstants('APPCENTER', ci, t)
 
-  delete process.env.APPCENTER
+  delete process.env.APPCENTER_BUILD_ID
+
+  t.end()
+})
+
+test('Codemagic - PR', function (t) {
+  process.env.CM_BUILD_ID = '1'
+  process.env.CM_PULL_REQUEST = '1'
+
+  clearModule('./')
+  const ci = require('./')
+
+  t.equal(ci.isCI, true)
+  t.equal(ci.isPR, true)
+  t.equal(ci.name, 'Codemagic')
+  t.equal(ci.CODEMAGIC, true)
+  assertVendorConstants('CODEMAGIC', ci, t)
+
+  delete process.env.CM_BUILD_ID
+  delete process.env.CM_PULL_REQUEST
+
+  t.end()
+})
+
+test('Codemagic - Not PR', function (t) {
+  process.env.CM_BUILD_ID = '1'
+
+  clearModule('./')
+  const ci = require('./')
+
+  t.equal(ci.isCI, true)
+  t.equal(ci.isPR, false)
+  t.equal(ci.name, 'Codemagic')
+  t.equal(ci.CODEMAGIC, true)
+  assertVendorConstants('CODEMAGIC', ci, t)
+
+  delete process.env.CM_BUILD_ID
+
+  t.end()
+})
+
+test('Xcode Cloud - PR', function (t) {
+  process.env.CI_XCODE_PROJECT = 'xx'
+  process.env.CI_PULL_REQUEST_NUMBER = '1'
+
+  clearModule('./')
+  const ci = require('./')
+
+  t.equal(ci.isCI, true)
+  t.equal(ci.isPR, true)
+  t.equal(ci.name, 'Xcode Cloud')
+  t.equal(ci.XCODE_CLOUD, true)
+  assertVendorConstants('XCODE_CLOUD', ci, t)
+
+  delete process.env.CI_XCODE_PROJECT
+  delete process.env.CI_PULL_REQUEST_NUMBER
+
+  t.end()
+})
+
+test('Xcode Cloud - Not PR', function (t) {
+  process.env.CI_XCODE_PROJECT = 'xx'
+
+  clearModule('./')
+  const ci = require('./')
+
+  t.equal(ci.isCI, true)
+  t.equal(ci.isPR, false)
+  t.equal(ci.name, 'Xcode Cloud')
+  t.equal(ci.XCODE_CLOUD, true)
+  assertVendorConstants('XCODE_CLOUD', ci, t)
+
+  delete process.env.CI_XCODE_PROJECT
+
+  t.end()
+})
+
+test('Xcode Server - Not PR', function (t) {
+  process.env.XCS = 'true'
+
+  clearModule('./')
+  const ci = require('./')
+
+  t.equal(ci.isCI, true)
+  // t.equal(ci.isPR, false)
+  t.equal(ci.name, 'Xcode Server')
+  t.equal(ci.XCODE_SERVER, true)
+  assertVendorConstants('XCODE_SERVER', ci, t)
+
+  delete process.env.XCS
 
   t.end()
 })
